from .abc import Serializer

# ---

from .S_BinStruct import *
from .S_Json import *
from .S_JsonPickle import *
from .S_Wrapping import *
from .S_Combination import *

# ---


from ._serializer import dumps, loads
