DATABASE_NAME = "SzQlib.db"
ENTRYPT_KEY = B"\x00\x02\xf3\x94\x98\x03\x99\xff\x00\x02\xf3\x94\x98\x03\x99\xff"
STRUCT_SAMPLE = {
    "name": "SzQlib",
    "version": "1.0.0",
    "author": "SzQ",
    "description": "A library for Python",
    "license": "MIT",
    "url": "",
    "email": "",
    "keywords": ["SzQlib", "SzQ", "Python"],
    "classifiers": [
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ]
}
